unit sumgrid;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Controls, StdCtrls, Grids;

type

  { TSumGrid }

  TSumGrid = class(TStringGrid)
  private
    procedure SetSetSumHorzVisible(AValue: boolean);
    procedure SetSumHorzHeight(AValue: integer);
    procedure SetSumVertVisible(AValue: boolean);
    procedure SetSumVertWidth(AValue: integer);
  protected
    //vGrSumHeight: integer;   //Store it as Variable to avoid if Assigned(fGrSum)...
    fSumHorz: TStringGrid;
    fSumHorzVisible: boolean;
    fSumHorzHeight: integer;    // das soll Höhe sein wenn es sichtbar ist
    vSumHorzHeight: integer;   // 0 bei fSumHorzVisible=false. Store it as Variable to avoid if Assigned(fGrSum)...

    fSumVert: TStringGrid;     // if fSumVertVisible=false then fSumVert:=nil;
    fSumVertVisible: boolean;
    fSumVertWidth: integer;    // das soll breite sein wenn es sichtbar ist
    vSumVertWidth: integer;    // 0 bei fSumVertVisible=false. Store it as Variable to avoid if Assigned(fGrSum)...

    procedure SumHorzCreate;
    procedure SumHorzUpdateProps;
    procedure SumHorzResize;

    procedure SumVertCreate;
    procedure SumVertUpdateProps;
    procedure SumVertResize;

    procedure SumHorzTopLeftChanged(Sender: TObject);
    procedure SumVertTopLeftChanged(Sender: TObject);

    //override
    function  GetClientRect: TRect; override;
    procedure Resize; override;
    procedure HeaderSized(IsColumn: Boolean; index: Integer); override;
    procedure TopLeftChanged; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property SumHorz: TStringGrid read fSumHorz stored false;
    property SumVert: TStringGrid read fSumVert stored false;
  published
    property SumHorzVisible: boolean read fSumHorzVisible write SetSetSumHorzVisible;

    property SumVertVisible: boolean read fSumVertVisible write SetSumVertVisible;
    property SumVertWidth: integer read fSumVertWidth write SetSumVertWidth;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ext', [TSumGrid]);
end;

{ TSumGrid }

{$REGION 'Create-Destroy' -fold}
constructor TSumGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Options:=Options-[goSmoothScroll];  //No half-scroll of columns

  fSumHorz:=nil;
  fSumHorzVisible:=false;
  fSumHorzHeight:=DefaultRowHeight;//+2;
  vSumHorzHeight:=0;

  fSumVert:=nil;
  fSumVertVisible:=false;
  fSumVertWidth:=DefaultColWidth;//+2;
  vSumHorzHeight:=0;
end;

destructor TSumGrid.Destroy;
begin
  if Assigned(fSumHorz) then FreeAndNil(fSumHorz);
  if Assigned(fSumVert) then FreeAndNil(fSumVert);
  inherited Destroy;
end;
{$ENDREGION}

{$REGION 'Overrided-Self-Methos' -fold}
function TSumGrid.GetClientRect: TRect;
begin
  Result:=inherited GetClientRect;
  Result.Bottom:=Result.Bottom-vSumHorzHeight;
  Result.Right:=Result.Right-vSumVertWidth;
end;

procedure TSumGrid.Resize;
begin
  inherited Resize;
  if Assigned(fSumHorz) then SumHorzResize;
  if Assigned(fSumVert) then SumVertResize;
end;

procedure TSumGrid.HeaderSized(IsColumn: Boolean; index: Integer);
var i: integer;
begin
  inherited HeaderSized(IsColumn, index);

  if IsColumn then begin
    if Assigned(fSumHorz) then begin
      if fSumHorz.ColCount<ColCount then  fSumHorz.ColCount:=ColCount;
      for i:=0 to ColCount-1 do
        fSumHorz.ColWidths[i]:=ColWidths[i];
    end;
  end
  else if Assigned(fSumVert) then begin
      if fSumVert.RowCount<RowCount then  fSumVert.RowCount:=RowCount;
      for i:=0 to RowCount-1 do
        fSumVert.RowHeights[i]:=RowHeights[i];
  end;
end;

procedure TSumGrid.TopLeftChanged;
begin
  inherited TopLeftChanged;
  //Called if user scrolling self --> synchronize fGrSum
  if Assigned(fSumHorz) then fSumHorz.LeftCol:=LeftCol;
  if Assigned(fSumVert) then fSumVert.TopRow:=TopRow;
end;
{$ENDREGION}

{$REGION 'SumHorz' -fold}
procedure TSumGrid.SetSetSumHorzVisible(AValue: boolean);
begin
  if fSumHorzVisible=AValue then Exit;
  fSumHorzVisible:=AValue;
  if (fSumHorzVisible=false) then begin
    vSumHorzHeight:=0;
    if Assigned(fSumHorz) then FreeAndNil(fSumHorz);
  end
  else if not Assigned(fSumHorz) then SumHorzCreate;
end;

procedure TSumGrid.SetSumHorzHeight(AValue: integer);
begin
  if fSumHorzHeight=AValue then Exit;
  fSumHorzHeight:=AValue;
end;

procedure TSumGrid.SumHorzCreate;
begin
  if Assigned(fSumHorz) then exit;
  vSumHorzHeight:=fSumHorzHeight;
  fSumHorz:= TStringGrid.Create(self);
   fSumHorz.OnTopLeftChanged:=@SumHorzTopLeftChanged;
   fSumHorz.Height:=fSumHorzHeight;
   fSumHorz.Parent:=self;
  SumHorzUpdateProps;
end;

procedure TSumGrid.SumHorzUpdateProps;
begin
  if not Assigned(fSumHorz) then exit;
  //Properties
  fSumHorz.BorderStyle:=bsNone;
  fSumHorz.FixedRows:=0;
  fSumHorz.RowCount:=1; //could be variable
  fSumHorz.FixedCols:=FixedCols;
  fSumHorz.DefaultColWidth:=DefaultColWidth;
  fSumHorz.DefaultRowHeight:=DefaultRowHeight;
  fSumHorz.ScrollBars:=ssNone;
end;

procedure TSumGrid.SumHorzTopLeftChanged(Sender: TObject);
begin
  //Called if user scrolling fSumHV --> synchronize self
  if Assigned(fSumHorz) and (LeftCol<>fSumHorz.LeftCol) then LeftCol:=fSumHorz.LeftCol;
end;

procedure TSumGrid.SumHorzResize;
var r: TRect;
begin
  //in Resize if not Assigned(fSumHorz) then exit;
  r:=GetClientRect;
   r.Top:=r.Bottom;
   r.Bottom:=ClientHeight;
  fSumHorz.SetBounds(r.Left, r.Top, r.Right, r.Bottom);
end;
{$ENDREGION}

{$REGION 'SumVert' -fold}
procedure TSumGrid.SetSumVertVisible(AValue: boolean);
begin
  if fSumVertVisible=AValue then Exit;
  fSumVertVisible:=AValue;
  if (fSumVertVisible=false) then begin
    vSumVertWidth:=0;
    if Assigned(fSumVert) then FreeAndNil(fSumVert);
  end
  else if not Assigned(fSumVert) then SumVertCreate;
end;

procedure TSumGrid.SetSumVertWidth(AValue: integer);
begin
  if fSumVertWidth=AValue then Exit;
  fSumVertWidth:=AValue;
end;

procedure TSumGrid.SumVertCreate;
begin
  if Assigned(fSumVert) then exit;
  vSumVertWidth:=fSumVertWidth;
  fSumVert:= TStringGrid.Create(self);
   fSumVert.OnTopLeftChanged:=@SumVertTopLeftChanged;
   fSumVert.Width:=fSumVertWidth;
   fSumVert.Parent:=self;
  SumVertUpdateProps;
end;

procedure TSumGrid.SumVertUpdateProps;
begin
  if not Assigned(fSumVert) then exit;
  //Properties
  fSumVert.BorderStyle:=bsNone;
  fSumVert.FixedRows:=FixedRows;
  fSumVert.ColCount:=1; //could be variable
  fSumVert.AutoFillColumns:=true;
  fSumVert.RowCount:=RowCount;
  fSumVert.FixedCols:=0;

  fSumVert.DefaultColWidth:=DefaultColWidth;
  fSumVert.DefaultRowHeight:=DefaultRowHeight;
  fSumVert.ScrollBars:=ssNone;
end;

procedure TSumGrid.SumVertTopLeftChanged(Sender: TObject);
begin
  //Called if user scrolling fSumHV --> synchronize self
  if Assigned(fSumVert) and (TopRow<>fSumVert.TopRow) then TopRow:=fSumVert.TopRow;
end;

procedure TSumGrid.SumVertResize;
var r: TRect;
begin
  //in Resize if not Assigned(fSumVert) then exit;
  r:=GetClientRect;
   r.Left:=r.Right;
   r.Right:=r.Left+vSumVertWidth;
   r.Bottom:=r.Bottom+vSumHorzHeight; //ClientHeight;
  fSumVert.SetBounds(r.Left, r.Top, r.Right, r.Bottom);
end;

{$ENDREGION}

end.

